/*
 * abaddrc.c - address-range database client for abmail
 *
 * Copyright (C) 2008 Taiji Yamada <taiji@aihara.co.jp>
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Affero General Public License for more details.
 */
/*
 * usage:
 *	$ ./abaddrc [-S server_host] [-f path_to.bdb] [-Z|-C|-R|-W|-s [-rc code] [-lt life_minutes] [-c comment] client_addr ...|-g client_addr ...]
 */
#include <stdio.h>	/* stderr, etc */
#include <string.h>	/* strcmp, etc */
#include <stdlib.h>	/* atoi, etc */
#include <arpa/inet.h>	/* inet_ntoa, inet_addr, inet_aton, etc */
#include <sys/types.h>	/* sys/socket.h */
#include <sys/socket.h>	/* AF_INET, etc */
#include "abcidr.h"
#include "abaddri.h"

#if !defined(HAVE_STRLCPY)
#define strlcpy(d, s, z) do { strncpy(d, s, z); if ((z) > 0) (d)[(z)-1] = '\0'; } while (0)
#endif

#if !defined(HAVE_STRLCAT)
#define strlcat(d, s, z) do { if ((z) > strlen(d)) strncat(d, s, (z)-strlen(d)-1); } while (0)
#endif

#if !defined(HAVE_INET_ATON)
#define inet_aton(s, p) ((((p)->s_addr=inet_addr(s))!=-1||strcmp(s, "255.255.255.255")==0)?1:0)
#endif

int main(int argc, char *argv[])
{
  int a, ct = 1;
  char *host = NULL, *file = NULL;
  int rc = -1, rv = 0;
  long lt = 0;
  char c[128] = "", *msg;

  am_init_masks();
#if !defined(DISABLE_IPv6)
  am6_init_masks();
#endif

  for (a=1; a<argc; a++) {
    if (strcmp(argv[a], "--") == 0)
      break;
    else if (strcmp(argv[a], "-S") == 0 && a+1 < argc)
      host = argv[++a];
    else if (strcmp(argv[a], "-f") == 0 && a+1 < argc)
      file = argv[++a];
    else if (strcmp(argv[a], "-Z") == 0) {
      rv = abaddr_req_serv(host, file, 0, argc, argv, &msg);
      if (rv != 0) fprintf(stderr, "%s\n", msg);
    }
    else if (strcmp(argv[a], "-C") == 0) {
      rv = abaddr_req_serv(host, file, 1, argc, argv, &msg);
      if (rv != 0) fprintf(stderr, "%s\n", msg);
    }
    else if (strcmp(argv[a], "-R") == 0) {
      rv = abaddr_req_serv(host, file, 2, argc, argv, &msg);
      if (rv != 0) fprintf(stderr, "%s\n", msg);
    }
    else if (strcmp(argv[a], "-W") == 0) {
      rv = abaddr_req_serv(host, file, 3, argc, argv, &msg);
      if (rv != 0) fprintf(stderr, "%s\n", msg);
    }
    else if (strcmp(argv[a], "-s") == 0)
      ct = 0;
    else if (strcmp(argv[a], "-rc") == 0 && a+1 < argc)
      rc = atoi(argv[++a]);
    else if (strcmp(argv[a], "-lt") == 0 && a+1 < argc) {
      int lm = atoi(argv[++a]);
      lt = (lm>0)?lm*60:lm;
    }
    else if (strcmp(argv[a], "-c") == 0 && a+1 < argc)
      strlcpy(c, argv[++a], sizeof(c));
    else if (strcmp(argv[a], "-g") == 0)
      ct = 1;
    else {
      char *p, *q;
      struct am am = { "", "", 0, NULL, };
#if !defined(DISABLE_IPv6)
      struct am6 am6 = { "", "", 0, NULL, };
#define am_set_space(a)		(af==AF_INET6)? am6_set_space(a##6):		(am_set_space)(a)
#define case_af(a6,a) ((af==AF_INET6)?(a6):(a))
#else
#define case_af(a6,a) (a)
#endif
      int af = AF_UNSPEC, f = AF_UNSPEC;
      int l;
      char *s, *e;

      p = argv[a];
      if ((q = strchr(p, '/'))) {
	*q = '\0';
	q++;
	strlcpy(am.a, p, sizeof(am.a));
	strlcpy(am.m, q, sizeof(am.m));
	if (inet_aton(p, &am.ads[0]) == 0) {
#if !defined(DISABLE_IPv6)
	  if (inet_pton(AF_INET6, p, &am6.ads[0]) == 0)
	    fprintf(stderr, "inet_pton failed for %s\n", p);
	  else {
	    strlcpy(am6.a, p, sizeof(am6.a));
	    strlcpy(am6.m, q, sizeof(am6.m));
	    af = AF_INET6;
	  }
#else
	  fprintf(stderr, "inet_aton failed for %s\n", p);
#endif
	}
	else
	  af = AF_INET;
	if ((l = am_set_space(&am)) == -1)
	  fprintf(stderr, "illegal cidr for %s/%s\n", p, q);
      }
      else if ((q = strchr(p, '-'))) {
	strlcpy(am.m, q, sizeof(am.m));
	*q = '\0';
	strlcpy(am.a, p, sizeof(am.a));
	if (inet_aton(p, &am.ads[0]) == 0) {
#if !defined(DISABLE_IPv6)
	  if (inet_pton(AF_INET6, p, &am6.ads[0]) == 0)
	    fprintf(stderr, "inet_pton failed for %s\n", p);
	  else {
	    strlcpy(am6.a, p, sizeof(am6.a));
	    strlcpy(am6.m, "-", sizeof(am6.m));
	    strlcat(am6.m, q+1, sizeof(am6.m));
	    af = AF_INET6;
	  }
#else
	  fprintf(stderr, "inet_aton failed for %s\n", p);
#endif
	}
	else
	  af = AF_INET;
	l = am_set_space(&am);
      }
      else {
	strlcpy(am.a, p, sizeof(am.a));
	if (inet_aton(p, &am.ads[0]) == 0) {
#if !defined(DISABLE_IPv6)
	  if (inet_pton(AF_INET6, p, &am6.ads[0]) == 0)
	    fprintf(stderr, "inet_pton failed for %s\n", p);
	  else {
	    strlcpy(am6.a, p, sizeof(am6.a));
	    af = AF_INET6;
	  }
#else
	  fprintf(stderr, "inet_aton failed for %s\n", p);
#endif
	}
	else
	  af = AF_INET;
	l = am_set_space(&am);
      }
      if (af == AF_UNSPEC)
	continue;
      switch (ct) {
      case 0:
	s = case_af((char *)&am6.ads[2], (char *)&am.ads[2]);
	e = case_af((char *)&am6.ads[3], (char *)&am.ads[3]);
	rv = abaddr_addr_set(host, file, af, s, e, rc, lt, c, &msg);
	if (rv != 0) fprintf(stderr, "%s\n", msg);
	break;
      case 1:
	s = case_af((char *)&am6.ads[2], (char *)&am.ads[2]);
	e = case_af((char *)&am6.ads[3], (char *)&am.ads[3]);
	rv = abaddr_addr_get(host, file, af, s, e, &f, &rc, &msg);
	if (rv != 0) fprintf(stderr, "%s\n", msg);
	else {
	  if (f != AF_UNSPEC)
	    fprintf(stdout, "%s\t%d\n", argv[a], rc);
	}
	break;
      }
      if (rv != 0)
	break;
    }
  }
  return rv;
}
