#!/bin/sh
#
# abackscatter.sh - anti-backscatter content filter
#
# Copyright (C) 2006-2008 Taiji Yamada <taiji@aihara.co.jp>
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Affero General Public License for more details.
#
MYDOMAIN=aihara.co.jp
HOME=/var/spool/postfix/filter; export HOME
LOG="$HOME/abackscatter.log"
LOCAL_RECIPIENT_MAPS="/etc/passwd /etc/mail/aliases"
LD_LIBRARY_PATH=/opt/local/lib; export LD_LIBRARY_PATH
SENDMAIL="/usr/lib/sendmail -i"

tmpf="$HOME/in.$$"
[ "$LOG" != "" ] && touch "$LOG"

flags=
while [ "$1" != "" ]; do
  case "$1" in
  --) shift; break;;
  *) flags="$flags $1";;
  esac
  shift
done

recipients=
scatters=
abmailrc=
if cat > "$tmpf"; then
  while [ "$1" != "" ]; do
    unknown=
    recipient=`echo "$1" | tr '[A-Z]' '[a-z]'`
    case "$recipient" in
    *$MYDOMAIN)
      user=`echo "$recipient" | sed -n 's/^\(.*\)@\(.*\)$/\1/p'`
      grep -i "^$user:" $LOCAL_RECIPIENT_MAPS > /dev/null
      [ "$?" != "0" ] && unknown=1
      ;;
    esac
    if [ "$unknown" = "1" ] && [ "$abmailrc" = "" ]; then
      /opt/local/bin/abmail -b -g -d < "$tmpf"
      abmailrc=$?
    fi
    if [ "$unknown" != "1" ] || [ "$abmailrc" != "1" ]; then
      recipients="$recipients $1"
    else
      scatters="$scatters $1"
    fi
    shift
  done
  if [ "$scatters" != "" ]; then
    [ "$LOG" != "" ] && echo "message content rejected:$flags --$scatters" >> "$LOG"
    code=0
  fi
  if [ "$recipients" != "" ]; then
    [ "$LOG" != "" ] && echo "message content accepted:$flags --$recipients" >> "$LOG"
    $SENDMAIL $flags -- $recipients < "$tmpf"
    code=$?
  fi
  rm -f "$tmpf"
else
  [ "$LOG" != "" ] && echo "temporary failure:$flags -- $@" >> "$LOG"
  code=75
fi
exit $code
